/*
 * molecule_template.hpp
 *
 *  Created on: Oct 6, 2009
 *      Author: heber
 */

#ifndef MOLECULE_TEMPLATE_HPP_
#define MOLECULE_TEMPLATE_HPP_

// ================== Acting on all Vectors ========================== //

// zero arguments
template <typename res> void molecule::ActOnAllVectors( res (Vector::*f)() )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)();
  }
};
template <typename res> void molecule::ActOnAllVectors( res (Vector::*f)() const )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)();
  }
};
template <typename res> void molecule::ActOnAllVectors( res (Vector::*f)() ) const
    {
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)();
  }
};
// one argument
template <typename res, typename T> void molecule::ActOnAllVectors( res (Vector::*f)(T), T t )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t);
  }
};
template <typename res, typename T> void molecule::ActOnAllVectors( res (Vector::*f)(T) const, T t )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t);
  }
};
template <typename res, typename T> void molecule::ActOnAllVectors( res (Vector::*f)(T), T t ) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t);
  }
};
// two arguments
template <typename res, typename T, typename U> void molecule::ActOnAllVectors( res (Vector::*f)(T, U), T t, U u )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t, u);
  }
};
template <typename res, typename T, typename U> void molecule::ActOnAllVectors( res (Vector::*f)(T, U) const, T t, U u )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t, u);
  }
};
template <typename res, typename T, typename U> void molecule::ActOnAllVectors( res (Vector::*f)(T, U), T t, U u ) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t, u);
  }
};
// three arguments
template <typename res, typename T, typename U, typename V> void molecule::ActOnAllVectors( res (Vector::*f)(T, U, V), T t, U u, V v)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t, u, v);
  }
};
template <typename res, typename T, typename U, typename V> void molecule::ActOnAllVectors( res (Vector::*f)(T, U, V) const, T t, U u, V v)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t, u, v);
  }
};
template <typename res, typename T, typename U, typename V> void molecule::ActOnAllVectors( res (Vector::*f)(T, U, V), T t, U u, V v) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    ((Walker->node)->*f)(t, u, v);
  }
};

// ================== Acting with each Atoms on same molecule ========================== //

// zero arguments
template <typename res> void molecule::ActWithEachAtom( res (molecule::*f)(atom *))
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (*f)(Walker);
  }
};
template <typename res> void molecule::ActWithEachAtom( res (molecule::*f)(atom *) const)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (*f)(Walker);
  }
};
template <typename res> void molecule::ActWithEachAtom( res (molecule::*f)(atom *)) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (*f)(Walker);
  }
};

// ================== Acting with each Atoms on copy molecule ========================== //

// zero arguments
template <typename res> void molecule::ActOnCopyWithEachAtom( res (molecule::*f)(atom *) , molecule *copy)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (copy->*f)(Walker);
  }
};
template <typename res> void molecule::ActOnCopyWithEachAtom( res (molecule::*f)(atom *) const , molecule *copy)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (copy->*f)(Walker);
  }
};
template <typename res> void molecule::ActOnCopyWithEachAtom( res (molecule::*f)(atom *) , molecule *copy) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (copy->*f)(Walker);
  }
};

// ================== Acting with each Atoms on copy molecule if true ========================== //

// zero arguments
template <typename res> void molecule::ActOnCopyWithEachAtomIfTrue( res (molecule::*f)(atom *) , molecule *copy, bool (atom::*condition) () )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    if ((Walker->*condition)())
      (copy->*f)(Walker);
  }
};
// one argument
template <typename res, typename T> void molecule::ActOnCopyWithEachAtomIfTrue( res (molecule::*f)(atom *) , molecule *copy, bool (atom::*condition) (T), T t )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    if ((Walker->*condition)(t))
      (copy->*f)(Walker);
  }
};
// two arguments
template <typename res, typename T, typename U> void molecule::ActOnCopyWithEachAtomIfTrue( res (molecule::*f)(atom *) , molecule *copy, bool (atom::*condition) (T, U), T t, U u )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    if ((Walker->*condition)(t,u))
      (copy->*f)(Walker);
  }
};
// three arguments
template <typename res, typename T, typename U, typename V> void molecule::ActOnCopyWithEachAtomIfTrue( res (molecule::*f)(atom *) , molecule *copy, bool (atom::*condition) (T, U, V), T t, U u, V v )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    if ((Walker->*condition)(t,u,v))
      (copy->*f)(Walker);
  }
};

// ================== Acting on all Atoms ========================== //

// zero arguments
template <typename res> void molecule::ActOnAllAtoms( res (atom::*f)())
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)();
  }
};
template <typename res> void molecule::ActOnAllAtoms( res (atom::*f)() const)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)();
  }
};
template <typename res> void molecule::ActOnAllAtoms( res (atom::*f)()) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)();
  }
};
// one argument
template <typename res, typename T> void molecule::ActOnAllAtoms( res (atom::*f)(T), T t )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t);
  }
};
template <typename res, typename T> void molecule::ActOnAllAtoms( res (atom::*f)(T) const, T t )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t);
  }
};
template <typename res, typename T> void molecule::ActOnAllAtoms( res (atom::*f)(T) const, T t ) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t);
  }
};
// two argument
template <typename res, typename T, typename U> void molecule::ActOnAllAtoms( res (atom::*f)(T, U), T t, U u )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u);
  }
};
template <typename res, typename T, typename U> void molecule::ActOnAllAtoms( res (atom::*f)(T, U) const, T t, U u )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u);
  }
};
template <typename res, typename T, typename U> void molecule::ActOnAllAtoms( res (atom::*f)(T, U), T t, U u ) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u);
  }
};
// three argument
template <typename res, typename T, typename U, typename V> void molecule::ActOnAllAtoms( res (atom::*f)(T, U, V), T t, U u, V v)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u, v);
  }
};
template <typename res, typename T, typename U, typename V> void molecule::ActOnAllAtoms( res (atom::*f)(T, U, V) const, T t, U u, V v)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u, v);
  }
};
template <typename res, typename T, typename U, typename V> void molecule::ActOnAllAtoms( res (atom::*f)(T, U, V), T t, U u, V v) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u, v);
  }
};
// four arguments
template <typename res, typename T, typename U, typename V, typename W> void molecule::ActOnAllAtoms( res (atom::*f)(T, U, V, W), T t, U u, V v, W w)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u, v, w);
  }
};
template <typename res, typename T, typename U, typename V, typename W> void molecule::ActOnAllAtoms( res (atom::*f)(T, U, V, W) const, T t, U u, V v, W w)
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u, v, w);
  }
};
template <typename res, typename T, typename U, typename V, typename W> void molecule::ActOnAllAtoms( res (atom::*f)(T, U, V, W), T t, U u, V v, W w) const
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (Walker->*f)(t, u, v, w);
  }
};

// ===================== Accesssing arrays indexed by some integer for each atom ======================

// for atom ints
template <typename T> void molecule::SetIndexedArrayForEachAtomTo ( T *array, int atom::*index, void (*Setor)(T *, T) )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (*Setor) (&array[(Walker->*index)], 1);
  }
};
template <typename T> void molecule::SetIndexedArrayForEachAtomTo ( T *array, int atom::*index, void (*Setor)(T *, T), T value )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (*Setor) (&array[(Walker->*index)], value);
  }
};
// for element ints inside atom class
template <typename T> void molecule::SetIndexedArrayForEachAtomTo ( T *array, int element::*index, void (*Setor)(T *, T) )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (*Setor) (&array[(Walker->type->*index)], 1);
  }
};
template <typename T> void molecule::SetIndexedArrayForEachAtomTo ( T *array, int element::*index, void (*Setor)(T *, T), T value )
{
  atom *Walker = start;
  while (Walker->next != end) {
    Walker = Walker->next;
    (*Setor) (&array[(Walker->type->*index)], value);
  }
};

#endif /* MOLECULE_TEMPLATE_HPP_ */
